@@ -1800,7 +1800,6 @@ TOOLS_VERSION_UINT_TO_COMPONENTS(const ToolsVersion toolsVersion,   // IN
 #define   TOOLS_VERSION_PLIERS_PATCH1_V_MNR  5
 #define   TOOLS_VERSION_PLIERS_PATCH1_V_BASE 1
 
-
 #ifndef RC_INVOKED
 #define   TOOLS_VERSION_PLIERS_PATCH2 TOOLS_VERSION_TO_UINT(TOOLS_VERSION_PLIERS_PATCH2_V)
 #endif /* RC_INVOKED */
@@ -1808,9 +1807,23 @@ TOOLS_VERSION_UINT_TO_COMPONENTS(const ToolsVersion toolsVersion,   // IN
 #define   TOOLS_VERSION_PLIERS_PATCH2_V_MNR  5
 #define   TOOLS_VERSION_PLIERS_PATCH2_V_BASE 2
 
-#define   TOOLS_VERSION_CURRENT        TOOLS_VERSION_PLIERS_PATCH2
-#define   TOOLS_VERSION_CURRENT_STR    TOOLS_VERSION_TO_STR(TOOLS_VERSION_PLIERS_PATCH2)
-#define   TOOLS_VERSION_CURRENT_CSV    TOOLS_VERSION_TO_CSV(TOOLS_VERSION_PLIERS_PATCH2)
+#ifndef RC_INVOKED
+#define   TOOLS_VERSION_PLIERS_PATCH3 TOOLS_VERSION_TO_UINT(TOOLS_VERSION_PLIERS_PATCH3_V)
+#endif /* RC_INVOKED */
+#define   TOOLS_VERSION_PLIERS_PATCH3_V_MJR  12
+#define   TOOLS_VERSION_PLIERS_PATCH3_V_MNR  5
+#define   TOOLS_VERSION_PLIERS_PATCH3_V_BASE 3
+
+#ifndef RC_INVOKED
+#define   TOOLS_VERSION_PLIERS_PATCH4 TOOLS_VERSION_TO_UINT(TOOLS_VERSION_PLIERS_PATCH4_V)
+#endif /* RC_INVOKED */
+#define   TOOLS_VERSION_PLIERS_PATCH4_V_MJR  12
+#define   TOOLS_VERSION_PLIERS_PATCH4_V_MNR  5
+#define   TOOLS_VERSION_PLIERS_PATCH4_V_BASE 4
+
+#define   TOOLS_VERSION_CURRENT        TOOLS_VERSION_PLIERS_PATCH4
+#define   TOOLS_VERSION_CURRENT_STR    TOOLS_VERSION_TO_STR(TOOLS_VERSION_PLIERS_PATCH4)
+#define   TOOLS_VERSION_CURRENT_CSV    TOOLS_VERSION_TO_CSV(TOOLS_VERSION_PLIERS_PATCH4)
 
 /*
  * The extended Tools version is the current Tools version with the
