@@ -1,5 +1,6 @@
 /*********************************************************
- * Copyright (C) 2008-2021 VMware, Inc. All rights reserved.
+ * Copyright (c) 2008-2024 Broadcom. All Rights Reserved.
+ * The term "Broadcom" refers to Broadcom Inc. and/or its subsidiaries.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU Lesser General Public License as published
@@ -59,6 +60,11 @@
 #include "err.h"
 #include "logToHost.h"
 #include "vthreadBase.h"
+#include "hostinfo.h"
+#include "vm_tools_version.h"
+#include "buildNumber.h"
+#include "vm_product.h"
+#include "util.h"
 
 #define LOGGING_GROUP         "logging"
 
@@ -120,6 +126,8 @@
 #define VMX_LOG_CMD     "log "
 #define VMX_LOG_CMD_LEN (sizeof(VMX_LOG_CMD) - 1)
 
+#define LOG_HEADER_MAX_ENTRIES 2
+
 
 typedef struct LogHandler {
    GlibLogger    *logger;
@@ -194,6 +202,9 @@ static RpcChannel *gChannel; /* either NULL or allocated AND started */
  */
 static GLogLevelFlags gLevelMask;
 
+static gchar *gLogHeaderBuf[LOG_HEADER_MAX_ENTRIES];
+static guint gLogHeaderCount;
+
 static enum RpcMode {
    RPC_OFF = 0,
    RPC_GUEST_LOG_TEXT,
@@ -212,6 +223,7 @@ static void LogWhereLevelV(LogWhere where,
                            const gchar *domain,
                            const gchar *fmt,
                            va_list args);
+static LogHandler *GetLogHandlerByDomain(const gchar *domain);
 
 /* Internal functions. */
 
@@ -496,6 +508,31 @@ VMToolsFreeLogEntry(gpointer data)
 }
 
 
+/**
+ * Function that logs a cached log header of Tools version, build details
+ * and Guest OS details.
+ *
+ * @param[in] _data     LogEntry pointer.
+ */
+
+static void
+VMToolsLogHeader(gpointer _data)
+{
+   LogEntry *entry = _data;
+   GlibLogger *logger = entry->handler->logger;
+   guint i;
+
+   for (i = 0; i < gLogHeaderCount; i++) {
+      char *message = VMToolsLogFormat(gLogHeaderBuf[i], entry->domain,
+                                       G_LOG_LEVEL_MESSAGE, entry->handler,
+                                       FALSE);
+
+      logger->logfn(entry->domain, G_LOG_LEVEL_MESSAGE, message, logger);
+      g_free(message);
+   }
+}
+
+
 /**
  * Function that calls the log handler.
  *
@@ -513,6 +550,10 @@ VMToolsLogMsg(gpointer _data, gpointer userData)
    gboolean usedSyslog = FALSE;
 
    if (logger != NULL) {
+       if (logger->logHeader) {
+          VMToolsLogHeader(entry);
+          logger->logHeader = FALSE;
+       }
        logger->logfn(entry->domain, entry->level, entry->msg, logger);
        usedSyslog = entry->handler->isSysLog;
    } else if (gErrorData->logger != NULL) {
@@ -1416,6 +1457,7 @@ VMToolsConfigLoggingInt(const gchar *defaultDomain,
    GPtrArray *oldDomains = NULL;
    LogHandler *oldDefault = NULL;
    GError *err = NULL;
+   char *gosDetails;
 
    g_return_if_fail(defaultDomain != NULL);
 
@@ -1489,6 +1531,34 @@ VMToolsConfigLoggingInt(const gchar *defaultDomain,
    gLogEnabled |= force;
    MarkLogInitialized();
 
+   /*
+    * Cache Tools version, build number and guest OS details.
+    * Cached log headers will be logged at log rotation and reset.
+    * No need to re-init the log headers in case of config reload.
+    */
+   if (gLogHeaderCount == 0) {
+      LogHandler *handler = GetLogHandlerByDomain(gLogDomain);
+      GlibLogger *logger = handler->logger;
+
+      logger->logHeader = TRUE;
+
+      gLogHeaderBuf[gLogHeaderCount++] = Str_Asprintf(NULL,
+                                                      "%s Version: %s (%s)",
+                                                      VMWARE_TOOLS_SHORT_NAME,
+                                                      TOOLS_VERSION_EXT_CURRENT_STR,
+                                                      BUILD_NUMBER);
+
+      gosDetails = Hostinfo_GetOSDetailedData();
+      if (gosDetails != NULL && gLogHeaderCount < LOG_HEADER_MAX_ENTRIES) {
+         gLogHeaderBuf[gLogHeaderCount++] = Str_Asprintf(NULL,
+                                                         "Guest OS details: %s",
+                                                         gosDetails);
+      }
+      free(gosDetails);
+
+      ASSERT(gLogHeaderCount <= LOG_HEADER_MAX_ENTRIES);
+   }
+
    gMaxCacheEntries = g_key_file_get_integer(cfg, LOGGING_GROUP,
                                              "maxCacheEntries", &err);
    if (err != NULL || gMaxCacheEntries < 0) {
@@ -2601,15 +2671,17 @@ VMTools_TeardownVmxGuestLog(void)
 
    /*
     * Acquire the same locks as VMTools_SetupVmxGuestLog.
+    * Stop the glib based logging to avoid nested logging
+    * from RpcChannel error.
     */
    VMTools_AcquireLogStateLock();
-
+   StopGlibLogging();
    g_rec_mutex_lock(&gVmxGuestLogMutex);
 
    DestroyRpcChannel();
 
    g_rec_mutex_unlock(&gVmxGuestLogMutex);
-
+   RestartGlibLogging();
    VMTools_ReleaseLogStateLock();
 }
 
@@ -2720,35 +2792,33 @@ VMTools_Log(LogWhere where,
 
 /*
  ******************************************************************************
- * VMTools_VmxLog --
+ * VMToolsVmxLogV --
  *
  * Sends the log message through RPC to vmx to be logged on the host.
  * Also, logs the message to vmsvc log file inside guest.
  *
  * @param[in]  chan         The RPC channel instance.
  * @param[in]  fmt          Log message output format.
+ * @param[in]  args         The argument list.
  *
  * @return None
  *
  ******************************************************************************
  */
 
-void
-VMTools_VmxLog(RpcChannel *chan,
+static void
+VMToolsVmxLogV(RpcChannel *chan,
                const gchar *fmt,
-               ...)
+               va_list args)
 {
    char *reply = NULL;
    size_t replyLen;
    gchar msg[4096] = VMX_LOG_CMD;
-   va_list args;
    gint len;
 
-   va_start(args, fmt);
    len = g_vsnprintf(msg + VMX_LOG_CMD_LEN,
                      sizeof msg - VMX_LOG_CMD_LEN,
                      fmt, args);
-   va_end(args);
 
    if (len <= 0) {
       g_warning("%s: g_vsnprintf failed: return value: %d.\n", __FUNCTION__,
@@ -2770,3 +2840,61 @@ VMTools_VmxLog(RpcChannel *chan,
 
    g_message("%s\n", msg + VMX_LOG_CMD_LEN);
 }
+
+
+/*
+ ******************************************************************************
+ * VMTools_VmxLog --
+ *
+ * Passes through VMToolsVmxLogV but takes the arguments inline.
+ *
+ * @param[in]  chan         The RPC channel instance.
+ * @param[in]  fmt          Log message output format.
+ *
+ * @return None
+ *
+ ******************************************************************************
+ */
+
+void
+VMTools_VmxLog(RpcChannel *chan,
+               const gchar *fmt,
+               ...)
+{
+   va_list args;
+
+   va_start(args, fmt);
+   VMToolsVmxLogV(chan, fmt, args);
+   va_end(args);
+}
+
+
+/*
+ ******************************************************************************
+ * VMTools_VmxLogThrottled --
+ *
+ * Passes through VMToolsVmxLogV to log the message after checking for the
+ * throttling condition. Takes the arguments inline.
+ *
+ * @param[in/out]  count     Throttle count.
+ * @param[in]      chan      The RPC channel instance.
+ * @param[in]      fmt       Log message output format.
+ *
+ * @return None
+ *
+ ******************************************************************************
+ */
+
+void
+VMTools_VmxLogThrottled(uint32 *count,
+                        RpcChannel *chan,
+                        const gchar *fmt,
+                        ...)
+{
+   va_list args;
+   if (Util_Throttle(++*count)) {
+      va_start(args, fmt);
+      VMToolsVmxLogV(chan, fmt, args);
+      va_end(args);
+   }
+}
