@@ -1,5 +1,6 @@
 /*********************************************************
- * Copyright (C) 2010-2015 VMware, Inc. All rights reserved.
+ * Copyright (c) 2010-2025 Broadcom. All Rights Reserved.
+ * The term "Broadcom" refers to Broadcom Inc. and/or its subsidiaries.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU Lesser General Public License as published
@@ -31,6 +32,16 @@
 #include "vixCommands.h"
 #include <glib.h>
 
+/*
+ * Needed for VGAuth support code.
+ */
+#ifdef _WIN32
+#include "VGAuthCommon.h"
+#include "VGAuthError.h"
+#include "VGAuthAuthentication.h"
+#include "VGAuthAuthenticationWin32.h"
+#include "VGAuthAlias.h"
+#endif
 
 #define PROCESS_CREATOR_USER_TOKEN       ((void *)1)
 
@@ -61,18 +72,28 @@ VixError VixTools_Initialize(Bool thisProcessRunsAsRootArg,
 
 void VixTools_Uninitialize(void);
 
-VixError VixToolsImpersonateUser(VixCommandRequestHeader *requestMsg, void **userToken);
+#ifdef _WIN32
+VixError VixToolsTranslateVGAuthError(VGAuthError vgErr);
+
+gchar *VixTools_ConfigGetString(const gchar *key,
+                                const gchar *defValue);
+
+void VixTools_ConfigLogInvalidString(const gchar *function,
+                                     const gchar *key,
+                                     const gchar *confValue,
+                                     const gchar *usedValue);
+
+#endif
+
+VixError VixToolsImpersonateUser(VixCommandRequestHeader *requestMsg,
+                                 Bool loadUserProfile,
+                                 void **userToken);
 
 void VixTools_SetConsoleUserPolicy(Bool allowConsoleUserOpsParam);
 
 void VixTools_SetRunProgramCallback(VixToolsReportProgramDoneProcType reportProgramDoneProc,
                                     void *clientData);
 
-gboolean VixTools_ConfigGetBoolean(GKeyFile *confDictRef,
-                                   const char *group,
-                                   const char *key,
-                                   gboolean defValue);
-
 void VixTools_RestrictCommands(gboolean restricted);
 
 /*
@@ -130,7 +151,7 @@ char *VixToolsGetEnvFromUserEnvironment(const VixToolsUserEnvironment *env,
 
 void VixToolsDestroyUserEnvironment(VixToolsUserEnvironment *env);
 
-VixError VixToolsValidateEnviron(char const * const *environ);
+VixError VixToolsValidateEnviron(char const * const *env);
 
 char *VixToolsGetEnvVarFromEnvBlock(const wchar_t *envBlock,
                                     const char *envVarName);
@@ -147,7 +168,7 @@ VixError VixToolsGetEnvBlock(void *userToken,
 
 Bool VixToolsDestroyEnvironmentBlock(wchar_t *envBlock);
 
-VixError VixToolsEnvironToEnvBlock(char const * const *environ,
+VixError VixToolsEnvironToEnvBlock(char const * const *env,
                                    wchar_t **envBlock);
 
 VixError VixToolsGetUserTmpDir(void *userToken,
@@ -187,6 +208,23 @@ VixError VixToolsListRegValuesImpl(VixCommandRequestHeader *requestMsg,
 
 VixError VixToolsDeleteRegValueImpl(VixCommandRequestHeader *requestMsg);
 
+gchar *VixToolsGetCurrentUsername(void);
+
+VixError VixToolsCheckSAMLForSystem(VGAuthContext *ctx,
+                                    VGAuthError origErr,
+                                    const char *token,
+                                    const char *username,
+                                    char *serviceUsername,
+                                    Bool hostVerified,
+                                    void **userToken,
+                                    VGAuthUserHandle **curUserHandle);
+
+VGAuthError
+VGAuth_ValidateUsernamePassword_Helper(VGAuthContext *ctx,
+                                       const char *userName,
+                                       const char *password,
+                                       VGAuthUserHandle **handle);
+
 #endif // _WIN32
 
 #ifdef VMX86_DEVEL
